;    music.inc, routines to play music
;    Copyright 2004 Chris Williams <cow3.14@juno.com>
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

SOUNDPORT	= 7
SOUNDOFF	= %11000000
SOUNDON		= %11111100

.module playsong
playsong:
	di			; is this necessary?
	
	ld	a,SOUNDOFF
	out	(SOUNDPORT),a
_getnote
	ld	a,(hl)
	or	a
	jr	z,_done
	
	inc	hl
	
	cp	86		; is it out of range?
	jr	nc,_nextnote
	ld	b,(hl)		; b = duration
	cp	85		; is it a pause?
	jr	nz,_notpause
	
	call	playpause	; do a pause of length B
	jr	_nextnote	; out of range
_notpause
	push	hl
	call	playnote	; play note A of length B
	pop	hl
_nextnote
	inc	hl
	jr	_getnote
_done
	ei
	ret

.module playpause
; pause for B ticks
playpause:
_pausetick
	push	bc
	
	ld	bc,2000		; magic number to pause for 1 tick
_pausesmall
	dec	c
	jr	nz,_pausesmall
	djnz	_pausesmall
	
	pop	bc
	djnz	_pausetick
	
	ret

.module playnote
; play note A for B ticks
playnote:
	ld	e,a
	ld	d,0		; de = note
	ld	a,b		; a = duration
	push	de
	
	; get half wavelength
	sla	e
	rl	d		; de *= 2
	ld	hl,half_wavelengths
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)		; de = half wavelength of note
	
	; get cycles
	pop	bc		; bc = note
	push	de		; save half wavelen
	ld	hl,cycles_per_tick
	add	hl,bc
	ld	l,(hl)		; l = cycles per tick
	ld	c,l		; save it
	dec	a		; this handles 0 (256)
	call	mulu		; multiply a * l unsigned
				; hl = ticks * cycles/tick = cycles
	ld	b,0		; bc = cycles per tick
	add	hl,bc		; this handles 0
;	ld	bc,7		; round up
;	add	hl,bc
	srl	h \	rr	l
	srl	h \	rr	l
	srl	h \	rr	l	; hl >> 3
	ld	b,h
	ld	c,l
	
	pop	de		; de = half wavelen
	; now we have BC = cycles and DE = half wavelength
	inc	b
	inc	c
	inc	d
	inc	e
	push	de
_cycle
	ld	d,b
	ld	e,c
	pop	bc		; bc = wavelength
	push	bc
	
	ld	a,SOUNDON
	out	(SOUNDPORT),a
	
_waveup
	nop \	nop
	dec	c
	jr	nz,_waveup
	djnz	_waveup

	pop	bc
	push	bc
	
	ld	a,SOUNDOFF
	out	(SOUNDPORT),a
_wavedown
	nop \	nop
	dec	c
	jr	nz,_wavedown
	djnz	_wavedown
	ld	b,d
	ld	c,e
	
	dec	c
	jr	nz,_cycle
	djnz	_cycle
	pop	de
	
	ret

.module noname
mulu:
#include "mul8x8.inc"

; *** NOTE ***
; I've modified the following tables from the generated tables for testing purposes

; these are the half wavelengths of each note
; these were generated by the program "wavelen" with freq = 214285 (6000000/28)
half_wavelengths = $ - 2
	.dw	1638*2,1546*2,1459*2,1377*2,1300*2,1227*2,1158*2,1093*2,1032*2, 974*2, 919*2, 868*2,
	.dw	 819*2, 773*2, 730*2, 689*2, 650*2, 614*2, 579*2, 547*2, 516*2, 487*2, 460*2, 434*2,
	.dw	 410*2, 387*2, 365*2, 344*2, 325*2, 307*2, 290*2, 273*2, 258*2, 244*2, 230*2, 217*2,
	.dw	 205*2, 193*2, 182*2, 172*2, 163*2, 153*2, 145*2, 137*2, 129*2, 122*2, 115*2, 108*2,
	.dw	 102*2,  97*2,  91*2,  86*2,  81*2,  77*2,  72*2,  68*2,  64*2,  61*2,  57*2,  54*2,
	.dw	  51*2,  48*2,  46*2,  43*2,  41*2,  38*2,  36*2,  34*2,  32*2,  30*2,  29*2,  27*2,
	.dw	  26*2,  24*2,  23*2,  22*2,  20*2,  19*2,  18*2,  17*2,  16*2,  15*2,  14*2,  14*2,
;	.dw	1638,1546,1459,1377,1300,1227,1158,1093,1032, 974, 919, 868,
;	.dw	 819, 773, 730, 689, 650, 614, 579, 547, 516, 487, 460, 434,
;	.dw	 410, 387, 365, 344, 325, 307, 290, 273, 258, 244, 230, 217,
;	.dw	 205, 193, 182, 172, 163, 153, 145, 137, 129, 122, 115, 108,
;	.dw	 102,  97,  91,  86,  81,  77,  72,  68,  64,  61,  57,  54,
;	.dw	  51,  48,  46,  43,  41,  38,  36,  34,  32,  30,  29,  27,
;	.dw	  26,  24,  23,  22,  20,  19,  18,  17,  16,  15,  14,  14,

; these are the number of wave cycles per tick (1/256 sec.) of each note
; this table is stored in 5:3 fixed-point format (5 bits for integral
; portion, 3 bits for fractional portion)
; these were generated by the program "cycles"
cycles_per_tick = $ - 1
	.db	  2/2,  2/2,  2/2,  2/2,  2/2,  3/2,  3/2,  3/2,  3/2,  3/2,  3/2,  4/2,
	.db	  4/2,  4/2,  4/2,  5/2,  5/2,  5/2,  5/2,  6/2,  6/2,  6/2,  7/2,  7/2,
	.db	  8/2,  8/2,  9/2,  9/2, 10/2, 10/2, 11/2, 12/2, 12/2, 13/2, 14/2, 15/2,
	.db	 15/2, 16/2, 17/2, 18/2, 19/2, 21/2, 22/2, 23/2, 24/2, 26/2, 28/2, 29/2,
	.db	 31/2, 33/2, 35/2, 37/2, 39/2, 41/2, 44/2, 46/2, 49/2, 52/2, 55/2, 58/2,
	.db	 62/2, 65/2, 69/2, 73/2, 78/2, 82/2, 87/2, 92/2, 98/2,104/2,110/2,117/2,
	.db	123/2,131/2,139/2,147/2,156/2,165/2,175/2,185/2,196/2,208/2,220/2,233/2,247/2,
;	.db	  2,  2,  2,  2,  2,  3,  3,  3,  3,  3,  3,  4,
;	.db	  4,  4,  4,  5,  5,  5,  5,  6,  6,  6,  7,  7,
;	.db	  8,  8,  9,  9, 10, 10, 11, 12, 12, 13, 14, 15,
;	.db	 15, 16, 17, 18, 19, 21, 22, 23, 24, 26, 28, 29,
;	.db	 31, 33, 35, 37, 39, 41, 44, 46, 49, 52, 55, 58,
;	.db	 62, 65, 69, 73, 78, 82, 87, 92, 98,104,110,117,
;	.db	123,131,139,147,156,165,175,185,196,208,220,233,247,

.end
